/*
  Java gfx-test framework

   Copyright (C) 2013, 2014  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;



import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.BitBltOperation;
import org.gfxtest.framework.annotations.BitBltOperations;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.RenderStyle;
import org.gfxtest.framework.annotations.RenderStyles;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



@TestType(TestTypes.RENDER_TEST)
@GraphicsPrimitive(GraphicsPrimitives.COMMON_BITMAP)
@RenderStyle(RenderStyles.NORMAL)
@BitBltOperation(BitBltOperations.BITBLT)
@Transformation(Transformations.SCALE)
@Zoom(1)
public class BitBltAffineScaleTransformOp extends BitBltAffineTransformOp
{
    private static double SCALES[] = {0.01, 0.1, 1/3., 0.5, 2/3., 1.0, 2.0, 3.0, 4.0};
    private static final int NUM_SCALES = SCALES.length * SCALES.length;

    private static final AffineTransform ScaleTransformation[] = new AffineTransform[NUM_SCALES];

    private static final RenderingHints RenderingHintsNearestNeighbor;
    private static final RenderingHints RenderingHintsBilinear;
    private static final RenderingHints RenderingHintsBicubic;

    private static AffineTransformOp ScaleTransformationNearest1Op[] = new AffineTransformOp[NUM_SCALES];
    private static AffineTransformOp ScaleTransformationBilinear1Op[] = new AffineTransformOp[NUM_SCALES];
    private static AffineTransformOp ScaleTransformationBicubic1Op[] = new AffineTransformOp[NUM_SCALES];

    private static AffineTransformOp ScaleTransformationNearest2Op[] = new AffineTransformOp[NUM_SCALES];
    private static AffineTransformOp ScaleTransformationBilinear2Op[] = new AffineTransformOp[NUM_SCALES];
    private static AffineTransformOp ScaleTransformationBicubic2Op[] = new AffineTransformOp[NUM_SCALES];

    static
    {
        RenderingHintsNearestNeighbor = new RenderingHints(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_NEAREST_NEIGHBOR);
        RenderingHintsBilinear = new RenderingHints(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        RenderingHintsBicubic = new RenderingHints(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);

        int i = 0;
        for (double verticalScale : SCALES) {
            for (double horizontalScale : SCALES) {
                ScaleTransformation[i] = AffineTransform.getScaleInstance(horizontalScale, verticalScale);

                ScaleTransformationNearest1Op[i] = new AffineTransformOp(ScaleTransformation[i], AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
                ScaleTransformationBilinear1Op[i] = new AffineTransformOp(ScaleTransformation[i], AffineTransformOp.TYPE_BILINEAR);
                ScaleTransformationBicubic1Op[i] = new AffineTransformOp(ScaleTransformation[i], AffineTransformOp.TYPE_BICUBIC);

                ScaleTransformationNearest2Op[i] = new AffineTransformOp(ScaleTransformation[i], RenderingHintsNearestNeighbor);
                ScaleTransformationBilinear2Op[i] = new AffineTransformOp(ScaleTransformation[i], RenderingHintsBilinear);
                ScaleTransformationBicubic2Op[i] = new AffineTransformOp(ScaleTransformation[i], RenderingHintsBicubic);

                i++;
            }
        }
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRScaleTransformation0Nearest1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, ScaleTransformationNearest1Op[0]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRScaleTransformation0Nearest2Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, ScaleTransformationNearest2Op[0]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRScaleTransformation0Bilinear1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, ScaleTransformationBilinear1Op[0]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRScaleTransformation0Bilinear2Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, ScaleTransformationBilinear2Op[0]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRScaleTransformation0Bicubic1Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, ScaleTransformationBicubic1Op[0]);
    }

    /**
     * Test basic BitBlt operation for checker buffered image with type TYPE_3BYTE_BGR.
     *
     * @param image
     *            image used as a destination for BitBlt-type operations
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testBitBltCheckerBufferedImageType3ByteBGRScaleTransformation0Bicubic2Op(TestImage image, Graphics2D graphics2d)
    {
        return doBitBltCheckerBufferedImageType3ByteRGB(image, graphics2d, ScaleTransformationBicubic2Op[0]);
    }

    /**
     * Entry point to the test suite.
     *
     * @param args not used in this case
     */
    public static void main(String[] args)
    {
        new BitBltAffineScaleTransformOp().runTestSuite(args);
    }

}
